#!/usr/bin/env python
import re
import sys
from operator import itemgetter
from exp_setting import *

VARIABLE_PREFIX = {}
VARIABLE_PREFIX[LIBSVM] = 'libsvm'
VARIABLE_PREFIX[SVM_LIGHT] = 'svm_light'
VARIABLE_PREFIX[SVM_RANK] = 'svm_rank'
VARIABLE_PREFIX[RV_SVM] = 'rv_svm'

def parse_time_log_file(file_path):
	records = {}
	with open(file_path, 'r') as f:
		for line in f.read().split('\n'):
			if len(line) == 0:continue
			slices = re.findall("iter ([0-9]+): time:([0-9]+\.[0-9]+)", line)[0]
			assert len(slices) == 2
			records[int(slices[0])] = slices[1]
	
	return records

def parse_performance_log_file(file_path):
	records = {}
	with open(file_path, 'r') as f:
		for line in f.read().split('\n'):
			if len(line) == 0:continue
			line = line.replace('-nan', '0.0')
			slices = re.findall("iter ([0-9]+): time:([0-9]+\.[0-9]+) obj:[0-9]+\.[0-9]+ apa:([0-9]+\.[0-9]+) meanndcg:([0-9]+\.[0-9]+) map:([0-9]+\.[0-9]+)", line)
			if len(slices) == 0:
				slices = re.findall("iter ([0-9]+): time:([0-9]+\.[0-9]+) apa:([0-9]+\.[0-9]+) meanndcg:([0-9]+\.[0-9]+) map:([0-9]+\.[0-9]+)", line)


			if len(slices) == 0:
				continue

			slices = slices[0]
			assert len(slices) == 5
			if float(slices[2]) == 100.0:
				continue
			records[int(slices[0])] = {
							"time" : slices[1],
							PA : slices[2],
							NDCG : slices[3],
							MAP : slices[4]
				}
	return records
	

def generate_plot_code(time_log_file_path, performance_log_file_path, variable_name_prefix = '', plot_option = '-k', rate = 0, thick = 4.0):
	time_records = parse_time_log_file(time_log_file_path)
	performance_records = parse_performance_log_file(performance_log_file_path)
	records = []

	for iteration in time_records:
		time = float(time_records[iteration])
		if iteration not in performance_records:
			continue

		performance = performance_records[iteration]
		records.append((time if time != 0.0 else 1.0, iteration, performance))

	sorted_records = sorted(records, key=itemgetter(0))

	partial_sorted_records = []
	last_iteration = -1
	for time, iteration, performance in records:
		if last_iteration != -1 and iteration - last_iteration < rate:
			continue
		last_iteration = iteration
		partial_sorted_records.append((time, performance))
	
	sorted_records = partial_sorted_records
	
	time_coordinates = [record[0] for record in sorted_records]
	apa_coordinates = [record[1][PA] for record in sorted_records]
	ndcg_coordinates = [record[1][NDCG] for record in sorted_records]
	map_cooridinates = [record[1][MAP] for record in sorted_records]

	time_coordinates += [100000]
	apa_coordinates += [apa_coordinates[-1]]
	ndcg_coordinates += [ndcg_coordinates[-1]]
	map_cooridinates += [map_cooridinates[-1]]

	code = {"declare":{}, "plot":{}}
	code["declare"]["time"] = '%s_time = [%s];' % (variable_name_prefix, ','.join(map(str, time_coordinates)))
	code["declare"][PA] = "%s_apa = [%s];" % (variable_name_prefix, ','.join(apa_coordinates))
	code["declare"][NDCG] = "%s_ndcg = [%s];" % (variable_name_prefix, ','.join(ndcg_coordinates))
	code["declare"][MAP] = "%s_map = [%s];" % (variable_name_prefix, ','.join(map_cooridinates))
	code["plot"][PA] = "semilogx(%s_time, %s_apa, '%s', 'LineWidth', %lf);" % (variable_name_prefix, variable_name_prefix, plot_option, thick)
	code["plot"][NDCG] = "semilogx(%s_time, %s_ndcg, '%s', 'LineWidth', %lf);" % (variable_name_prefix, variable_name_prefix, plot_option, thick)
	code["plot"][MAP] = "semilogx(%s_time, %s_map, '%s', 'LineWidth', %lf);" % (variable_name_prefix, variable_name_prefix, plot_option, thick)

	
	return code

def add_single_quote(s):
	return "'" + s + "'"
def get_target_decription(target):
	return TARGET_DESCRIPTION[target]

def main():
	for dataset, targets in TASK_LIST:
		print "generating", dataset
		#get plot code
		plot_codes = {}
		for target in targets:
			plot_codes[target] = {}
			for metric in TARGET_METRIC[dataset]:
				time_log_file_path = EXP_LOG_DIR + '/' + '.'.join([dataset, target, metric, 'timelog'])
				performance_log_file_path = EXP_LOG_DIR + '/' + '.'.join([dataset, target, metric, 'log'])
				code = generate_plot_code(time_log_file_path, performance_log_file_path,
														VARIABLE_PREFIX[target], PLOT_OPTION[target],
														50 if target == SVM_LIGHT else 0,
														5.5 if target == SVM_RANK or target == LIBSVM else 3.5
														)
				plot_codes[target][metric] = '\n'.join([code["declare"]["time"], code["declare"][metric], code["plot"][metric]])


		#generate matlab file
		for metric in TARGET_METRIC[dataset]:
			with open(PLOT_DIR + '/' + '_'.join([dataset.replace('-','_'), metric]) + '.m', 'w') as f:
				print >> f, "h = figure;"
				hold_on = False
				for target in targets:
					code = plot_codes[target][metric]
					print >> f, code
					if not hold_on:
						hold_on = True
						print >> f, "hold on;"
				print >> f, "xlabel('Time (sec.)', 'FontSize', 25);"
				print >> f, "ylabel('%s', 'FontSize', 25);" % (METRIC_DESCIPTION[metric])
				print >> f, "set(gca, 'FontSize', 20);"
				print >> f, "legend(%s, 'location', 'SouthWest');" % (','.join(map(add_single_quote, map(get_target_decription, targets))))
				print >> f, "xlim([%d %d]);" % (THRESHOLD[dataset], MAX[dataset])
				print >> f, "saveas(h, './figures/%s_%s.eps', 'epsc');" % (dataset, metric)
				print >> f, "saveas(h, './figures/%s_%s.pdf', 'pdf');" % (dataset, metric)
			


if __name__ == '__main__':
	main()
	

